function Initialize()
    if global.initialized == nil then
        global.initialized = true
    end

    global.playerInitialized = {}

    for _, player in pairs(game.players) do
        SetupPlayer(player)
    end
end

function SetupPlayer(player)
    global.playerInitialized[player.name] = 1
    global.checkboxState = {}
    global.checkboxState.box1 = false
    global.checkboxState.box2 = false
    global.checkboxState.box3 = false
    global.checkboxState.box4 = false

    local topBar = player.gui.top.add{
        type="frame",
        name="quickInfo",
        direction = "horizontal"}

    topBar.style.minimal_height = 30
    topBar.style.minimal_width = 200

    topBar.add{
        type = "sprite-button",
        name = "ShowWindow",
        sprite = "entity/player",
        tooltip = {"character-screen-button"}
    }
end

function onPlayerJoin(event)
    local player = game.players[event.player_index]
    if not (global.playerInitialized[player.name]) then
        SetupPlayer(player)
    end
end

local function on_gui_click(event)
    local name = event.element.name
    local player = game.players[event.element.player_index]
    if (name == "ShowWindow") then
        ShowDemoWindow(player)
    elseif string.sub(name, 1, 8) == 'checkbox' then
        SetCheckboxState(player, name)
    end
end

function ShowDemoWindow(player)
    local frame = player.gui.center["checkbox-demo"]
    if frame then
        frame.destroy()
        return
    end

    frame = player.gui.center.add{
        type = "frame",
        name = "checkbox-demo",
        direction = "vertical",
    }

    frame.style.minimal_height = 200
    frame.style.minimal_width = 400

    frame.add {
        type = "checkbox",
        name = "checkbox1",
        state = global.checkboxState.box1,
    }

    frame.add {
        type = "checkbox",
        name = "checkbox2",
        state = global.checkboxState.box2,
    }

    frame.add {
        type = "checkbox",
        name = "checkbox3",
        state = global.checkboxState.box3,
    }

    frame.add {
        type = "checkbox",
        name = "checkbox4",
        state = global.checkboxState.box4,
    }

    UpdateCheckboxState(frame)
end

function SetCheckboxState(player, name)
    local frame = player.gui.center["checkbox-demo"]

    if name == "checkbox1" then
        global.checkboxState.box1 = true
    elseif name == "checkbox2" then
        global.checkboxState.box2 = true
    elseif name == "checkbox3" then
        global.checkboxState.box3 = true
    elseif name == "checkbox4" then
        global.checkboxState.box4 = true
    end

    UpdateCheckboxState(frame)
end

function UpdateCheckboxState(frame)
    frame.checkbox1.enabled = (global.checkboxState.box1 == false)
    frame.checkbox2.enabled = (global.checkboxState.box1 == true and global.checkboxState.box2 == false)
    frame.checkbox3.enabled = (global.checkboxState.box2 == true and global.checkboxState.box3 == false)
    frame.checkbox4.enabled = (global.checkboxState.box3 == true and global.checkboxState.box4 == false)
end

script.on_init(Initialize)
script.on_configuration_changed(Initialize)
script.on_event(defines.events.on_player_joined_game, onPlayerJoin)
script.on_event(defines.events.on_player_created, onPlayerJoin)
script.on_event(defines.events.on_gui_click, on_gui_click)
script.on_event("open-demo-window", function(event) ShowDemoWindow(game.players[event.player_index]) end)